﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;

namespace XamDataChartMarkersVisSL
{
    public class CustomLineSeries : HorizontalAnchoredCategorySeries
    {
        private Canvas _markerCanvas = new Canvas();

        public CustomLineSeries()
        {
            this.DefaultStyleKey = typeof(CustomLineSeries);
        }

        protected override void RenderSeriesOverride(bool animate)
        {
            base.RenderSeriesOverride(animate);

            if (this.Viewport.IsEmpty || this.RootCanvas == null ||
    this.XAxis == null || this.YAxis == null ||
    this.ItemsSource == null)
            {
                return;
            }

            this.RootCanvas.Children.Clear();

            PointCollection points = new PointCollection();

            double index = 0;
            //Getting Points based on the ItemsSource
            foreach (var item in ItemsSource)
            {
                //Getting X Value
                double xValue = 0;

                if (XAxis is CategoryXAxis)
                    xValue = index;

                if (XAxis is CategoryDateTimeXAxis)
                {
                    CategoryDateTimeXAxis xAxis = XAxis as CategoryDateTimeXAxis;
                    DateTime dateValue = (DateTime)item.GetType().GetProperty(xAxis.DateTimeMemberPath).GetValue(item, null);
                    xValue = dateValue.Ticks;
                }

                //Getting Y Value
                double yValue = 0;
                object yValObj = item.GetType().GetProperty(ValueMemberPath).GetValue(item, null);

                if (yValObj != null)
                    yValue = Convert.ToDouble(yValObj);

                ScalerParams xParams = new ScalerParams(this.SeriesViewer.WindowRect, this.Viewport, XAxis.IsInverted);
                ScalerParams yParams = new ScalerParams(this.SeriesViewer.WindowRect, this.Viewport, YAxis.IsInverted);
                double x = this.XAxis.GetScaledValue(xValue, xParams);
                double y = this.YAxis.GetScaledValue(yValue, yParams);

                points.Add(new Point(x, y));

                index++;

            }

            //Creating Simple Markers 
            _markerCanvas.Children.Clear();
            foreach (Point point in points)
            {
                Ellipse marker = new Ellipse()
                {
                    Fill = this.ActualBrush,
                    Width = 15,
                    Height = 15,
                };

                Canvas.SetTop(marker, point.Y - 7.5);
                Canvas.SetLeft(marker, point.X - 7.5);

                _markerCanvas.Children.Add(marker);
            }

            Polyline line = new Polyline();

            line.Points = points;
            line.Stroke = this.ActualBrush;
            line.StrokeThickness = this.Thickness;

            this.RootCanvas.Children.Add(line);
            if (!RootCanvas.Children.Contains(_markerCanvas))
                this.RootCanvas.Children.Add(_markerCanvas);

        }

        public bool UseNormalMode
        {
            get { return (bool)GetValue(UseNormalModeProperty); }
            set { SetValue(UseNormalModeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for UseNormalMode.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty UseNormalModeProperty =
            DependencyProperty.Register("UseNormalMode",
            typeof(bool),
            typeof(CustomLineSeries),
            new PropertyMetadata(false,
                (o, e) =>
                {
                    (o as CustomLineSeries).RenderSeriesOverride(false);
                }));
    }
}
